    processor 6502
    include vcs.h
    include macro.h
    include music.h

; Compile switches

NTSC                    = 0
PAL                     = 1 
COMPILE_VERSION         = NTSC

; Equates

BALLOONFRAMES           = 14
KERNEL_HEIGHT           = $9C
BALLOON_HEIGHT          = 19
PF_LINES                = 26
PF_SIZE                 = 3
VERTBOTTOMLIMIT         = BALLOON_HEIGHT
VERTTOPLIMIT            = KERNEL_HEIGHT

; Variables

    SEG.U vars
    ORG $80

caveshape1          ds PF_LINES
caveshape2          ds PF_LINES
caveshape3          ds PF_LINES
caveshape4          ds PF_LINES
verPosBalloon       ds 1
horPosBalloon       ds 1
frameCounter        ds 1
balloonAnim         ds 1
balloonPtr          ds 2
balloonmovPtr       ds 2
balloonOffset       ds 1
PFCounter           ds 1
shiftval            ds 1

    SEG     Bank0
    ORG $F800

;#######################
;# MAIN SCREEN DISPLAY #
;#######################

SkipBalloon
                LDA #$00
                SLEEP 7
                BEQ BalloonContinue
SkipBalloon2
                LDA #$00
                SLEEP 7
                BEQ BalloonContinue2

; Start display kernel
Maindisplay
        LDY shiftval
        LDX shifttab,Y
ShiftMore
        STA WSYNC
        STA VBLANK          ; Stop vertical blank
        DEX
        BPL ShiftMore
 

        LDA #$FF
        STA PF1
        STA PF2
        STA HMCLR
        LDY #KERNEL_HEIGHT
        LDX #PF_LINES-1
        LDA #$00
        STA PFCounter
        STA WSYNC
        STA PF1
        STA PF2
load    
        LDA #BALLOON_HEIGHT-1
        DCP balloonOffset
        BCC SkipBalloon
        LDA (balloonmovPtr),Y
        STA HMP0
        LDA (balloonPtr),Y
BalloonContinue
        STA WSYNC
        STA HMOVE
        STA GRP0
        LDX divideby3,Y
        LDA caveshape1,X
        STA PF1
        LDA caveshape2,X
        STA PF2
        DEY
        LDA #BALLOON_HEIGHT-1
        DCP balloonOffset

        SLEEP 8

        LDA caveshape3,X
        STA PF2
        LDA caveshape4,X
        STA PF1
        BCC SkipBalloon2
        LDA (balloonmovPtr),Y
        STA HMP0
        LDA (balloonPtr),Y
BalloonContinue2
        STA WSYNC
        STA HMOVE
        STA GRP0
        LDA #$00
        STA PF1
        STA PF2

        SLEEP 32

        DEY
        BNE  load      ;get ready for next scanline

        STA WSYNC
        STY GRP0

        LDA #$FF
        STA PF1
        STA PF2

        LDA shiftval
        EOR #$08
        TAY
        LDX shifttab,Y
ShiftMore2
        STA WSYNC
        LDA #$00
        STA PF1
        STA PF2
        DEX
        BPL ShiftMore2


        REPEAT 23
        STA WSYNC
        REPEND

        LDA horPosBalloon
        LDX #$00
        JSR PosElement        
        
        LDX  #$1E
        JMP Overscan

;########################
;# CARTRIDGE MAIN ENTRY #
;########################

Start
    CLEAN_START
    JSR SetStartPos
    LDA #$32           ;
    STA COLUP0         ; Sprite colour
    LDA #$AA
    STA COLUPF         ; Sprite colour
    LDA #$01           ;
    STA CTRLPF

    SET_POINTER balloonPtr, BALLOON18
    SET_POINTER balloonmovPtr, BALLOONM

    LDX #PF_LINES-1
CreateCave
    LDA BALLOONLEVEL0+0*PF_LINES,X
    STA caveshape1,X
    LDA BALLOONLEVEL0+1*PF_LINES,X
    STA caveshape2,X
    LDA BALLOONLEVEL0+2*PF_LINES,X
    STA caveshape3,X
    LDA BALLOONLEVEL0+3*PF_LINES,X
    STA caveshape4,X
    DEX
    BPL CreateCave
    

; Main Program Loop

NEWSCREEN
    VERTICAL_SYNC

; Set VBLANK timer
    IF COMPILE_VERSION = PAL
        LDA #73
    ELSE
        LDA #43
    ENDIF
    STA TIM64T

    LDX balloonAnim
    LDA #KERNEL_HEIGHT
    SEC
    SBC verPosBalloon
    ADC #BALLOON_HEIGHT-1
    STA balloonOffset
    CLC
    ADC balloonAnimTab,X
    STA balloonPtr
    LDA balloonOffset
    CLC
    ADC balloonMoveTab,X
    STA balloonmovPtr
    LDA balloonMoveTabHigh,X
    STA balloonmovPtr+1

VBLANKLOOP
    LDA INTIM
    BNE VBLANKLOOP  ; WAIT FOR VBLANK TIMER
    JMP Maindisplay

Overscan

; Set Overscan timer
    IF COMPILE_VERSION = PAL
        LDA #65
    ELSE
        LDA #36
    ENDIF
    STA TIM64T

; Collission Detection
    LDA CXP0FB
    ASL
    BCC NoCol
    JSR SetStartPos
NoCol    
    STA CXCLR

; Frame Stuff
    INC frameCounter
    LDA frameCounter
    AND #$01
    BNE NoJoy
    JSR joy        ;overscan. Time to read the joysticks.
NoJoy

    LDA frameCounter
    LSR
    LSR
    LSR
    LSR
    AND #$0F
    CMP shiftval
    BEQ NoChange
    STA shiftval
    TAX
    LDA shiftaddtab,X
    CLC
    ADC verPosBalloon
    STA verPosBalloon
NoChange
  
; Limit vertical movement
    LDA verPosBalloon
    CMP #VERTBOTTOMLIMIT
    BCS PosFine1
    LDA #VERTBOTTOMLIMIT
PosFine1
    CMP #VERTTOPLIMIT+1
    BCC PosFine2
    LDA #VERTTOPLIMIT
PosFine2
    STA verPosBalloon
  
    LDA frameCounter
    AND #$07
    BNE OVERSCANLOOP

; Balloon Animation
    INC balloonAnim
    LDA balloonAnim
    EOR #BALLOONFRAMES
    BNE OVERSCANLOOP
    STA balloonAnim

OVERSCANLOOP
    LDA INTIM
    BNE OVERSCANLOOP    ; WAIT FOR OVERSCAN TIMER
    STA WSYNC       ; FINISH WAITING FOR THE CURRENT LINE

    JMP NEWSCREEN

joy     
        LDA SWCHA      
        ASL
        BCS RightDone
        INC horPosBalloon
RightDone
        ASL
        BCS LeftDone
        DEC horPosBalloon
LeftDone
        ASL
        BCS DownDone
        DEC verPosBalloon
DownDone
        ASL
        BCS UpDone
        INC verPosBalloon
UpDone
        RTS

    ORG $FA00

divideby3
    .byte 0,0
    .byte 0,0,0
    .byte 0,0,0
    .byte 1,1,1
    .byte 1,1,1
    .byte 2,2,2
    .byte 2,2,2
    .byte 3,3,3
    .byte 3,3,3
    .byte 4,4,4
    .byte 4,4,4
    .byte 5,5,5
    .byte 5,5,5
    .byte 6,6,6
    .byte 6,6,6
    .byte 7,7,7
    .byte 7,7,7
    .byte 8,8,8
    .byte 8,8,8
    .byte 9,9,9
    .byte 9,9,9
    .byte 10,10,10
    .byte 10,10,10
    .byte 11,11,11
    .byte 11,11,11
    .byte 12,12,12
    .byte 12,12,12
    .byte 13,13,13
    .byte 13,13,13
    .byte 14,14,14
    .byte 14,14,14
    .byte 15,15,15
    .byte 15,15,15
    .byte 16,16,16
    .byte 16,16,16
    .byte 17,17,17
    .byte 17,17,17
    .byte 18,18,18
    .byte 18,18,18
    .byte 19,19,19
    .byte 19,19,19
    .byte 20,20,20
    .byte 20,20,20
    .byte 21,21,21
    .byte 21,21,21
    .byte 22,22,22
    .byte 22,22,22
    .byte 23,23,23
    .byte 23,23,23
    .byte 24,24,24
    .byte 24,24,24
    .byte 25,25,25
    .byte 25,25

    ORG $FC00
BALLOONLEVEL0
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111110
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111100
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000

                                .byte   %11111111
                                .byte   %00000111
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00001100
                                .byte   %11111100
                                .byte   %11100000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %10000001
                                .byte   %11000011
                                .byte   %01111100
                                .byte   %00111000
                                .byte   %00010000
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000011
                                .byte   %00000000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %11100000
                                .byte   %11111000

                                .byte   %10000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000111
                                .byte   %00011111
                                .byte   %11111110
                                .byte   %11111100
                                .byte   %00011000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000010
                                .byte   %00111100
                                .byte   %00111100
                                .byte   %00001100
                                .byte   %00000100
                                .byte   %00000010
                                .byte   %00000000
                                .byte   %10000000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %11111100
                                .byte   %11111111

                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11100000
                                .byte   %11111111
                                .byte   %11111111
                                .byte   %11111110
                                .byte   %11111100
                                .byte   %11111100
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11110000
                                .byte   %11111000
                                .byte   %11111110
                                .byte   %11111111

    ORG $FD00

PosElement
       SEC
       STA WSYNC                ;
.waitzone
       SBC #$0F                 ;
       BCS .waitzone            ; RESP loop
       EOR #$07                 ;
       ASL                      ;
       ASL                      ;
       ASL                      ;
       ASL                      ;
       STA HMP0,X
       STA RESP0,X           ; Position it!
       STA WSYNC
       STA HMOVE
       RTS                      ; done, that's all!

SetStartPos
    LDA #$14
    STA verPosBalloon
    LDA #$78
    STA horPosBalloon
    RTS

    ORG $FD90

BALLOONLLL
    .byte $00
    .byte $00
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $50
    .byte $00

BALLOONLL
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $F0
    .byte $00
    .byte $F0
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $40
    .byte $00

    ORG $FE90

BALLOONL
    .byte $00
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $30
    .byte $00

BALLOONM
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

BALLOONR
    .byte $00
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $D0
    .byte $00

BALLOONRR
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $10
    .byte $00
    .byte $10
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $C0
    .byte $00

BALLOONRRR
    .byte $00
    .byte $00
    .byte $10
    .byte $10
    .byte $10
    .byte $10
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $B0
    .byte $00

shifttab
        .byte 0,1,2,3,4,5,6,7,8,7,6,5,4,3,2,1

shiftaddtab
        .byte -1,1,1,1,1,1,1,1,1,-1,-1,-1,-1,-1,-1,-1

    ORG $FF80

balloonMoveTabHigh
    .byte >BALLOONLLL,>BALLOONLLL
    .byte >BALLOONLL,>BALLOONL
    .byte >BALLOONM,>BALLOONR
    .byte >BALLOONRR,>BALLOONRRR
    .byte >BALLOONRRR,>BALLOONRR
    .byte >BALLOONR,>BALLOONM
    .byte >BALLOONL,>BALLOONLL

balloonMoveTab
    .byte <BALLOONLLL - KERNEL_HEIGHT - 1, <BALLOONLLL - KERNEL_HEIGHT - 1
    .byte <BALLOONLL - KERNEL_HEIGHT - 1, <BALLOONL - KERNEL_HEIGHT - 1
    .byte <BALLOONM - KERNEL_HEIGHT - 1, <BALLOONR - KERNEL_HEIGHT - 1
    .byte <BALLOONRR - KERNEL_HEIGHT - 1, <BALLOONRRR - KERNEL_HEIGHT - 1
    .byte <BALLOONRRR - KERNEL_HEIGHT - 1, <BALLOONRR - KERNEL_HEIGHT - 1
    .byte <BALLOONR - KERNEL_HEIGHT - 1, <BALLOONM - KERNEL_HEIGHT - 1
    .byte <BALLOONL - KERNEL_HEIGHT - 1, <BALLOONLL - KERNEL_HEIGHT - 1

balloonAnimTab
    .byte <BALLOON14 - KERNEL_HEIGHT - 1, <BALLOON14 - KERNEL_HEIGHT - 1
    .byte <BALLOON16 - KERNEL_HEIGHT - 1, <BALLOON17 - KERNEL_HEIGHT - 1
    .byte <BALLOON18 - KERNEL_HEIGHT - 1, <BALLOON17 - KERNEL_HEIGHT - 1
    .byte <BALLOON16 - KERNEL_HEIGHT - 1, <BALLOON14 - KERNEL_HEIGHT - 1
    .byte <BALLOON14 - KERNEL_HEIGHT - 1, <BALLOON16 - KERNEL_HEIGHT - 1
    .byte <BALLOON17 - KERNEL_HEIGHT - 1, <BALLOON18 - KERNEL_HEIGHT - 1
    .byte <BALLOON17 - KERNEL_HEIGHT - 1, <BALLOON16 - KERNEL_HEIGHT - 1

BALLOON18
        .byte %00010000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00010000

BALLOON17
        .byte %00010000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00010000
        .byte %00000000

BALLOON16
        .byte %00010000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00010000
        .byte %00000000
        .byte %00000000

BALLOON14
        .byte %00010000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %01111100
        .byte %00111000
        .byte %00010000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000

    ORG $FFFA
    .WORD Start
    .WORD Start
    .WORD Start